package org.hso.ecommerce.controller.intern.customers;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import org.hibernate.Session;
import org.hso.ecommerce.action.shop.RestServiceForDelivery;
import org.hso.ecommerce.entities.shop.CustomerOrder;
import org.hso.ecommerce.repos.shop.CustomerOderRepository;
import org.hso.ecommerce.repos.user.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Controller
@RequestMapping("intern/customerOrders")
public class CustomerOrderController {

    @Autowired
    private final CustomerOderRepository customerOrderRepository = null;

    @GetMapping("")
    public String internCustomerOrder(Model model) {
        List<CustomerOrder> orders =  customerOrderRepository.getAllOrders();

        Map<CustomerOrder, DeliveryData> customerOrderDeliveryDataMap = orders.stream().collect
                (Collectors.toMap
                (Function.identity(), this::getDeliveryDataFromCustomerOrder));

        model.addAttribute("orderDeliveryDataMap", customerOrderDeliveryDataMap);
        return "intern/customerOrders/index";
    }

    @GetMapping("/{id}")
    public String internCustomerOrdersId(Model model,
                                         @PathVariable("id") String id
    ) {
        CustomerOrder order = customerOrderRepository.findById(Long.parseLong(id)).get();
        DeliveryData deliveryData = getDeliveryDataFromCustomerOrder(order);

        model.addAttribute("order", order);
        model.addAttribute("deliveryData", deliveryData);

        return "intern/customerOrders/id";
    }

    private  DeliveryData getDeliveryDataFromCustomerOrder(CustomerOrder customerOrder)
    {
        if(customerOrder.trackingId == null)
            return new CustomerOrderController.DeliveryData("Bestellung wurde elektronisch angekündigt","");

        if(customerOrder.deliveredAt == null)
        {
            DeliveryData deliveryData = new RestServiceForDelivery(new RestTemplateBuilder()).getDeliveryData(UUID.fromString(customerOrder.trackingId));

            if(deliveryData.isDelivered())
            {
                customerOrderRepository.updateUserDeliveredAt(customerOrder.id, Timestamp.valueOf(deliveryData.estimatedArrival));
                System.out.println(Timestamp.valueOf(deliveryData.estimatedArrival));
            }

            return deliveryData;
        }

        SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-yyyy");
        return new CustomerOrderController.DeliveryData("Lieferung erfolgreich",formatter.format(customerOrder.deliveredAt));
    }

    public static class DeliveryData
    {
        @JsonProperty("status")
        private String status;
        @JsonProperty("estimatedArrival")
        private String estimatedArrival;

        private boolean isDelivered;

        @JsonCreator(mode = JsonCreator.Mode.DELEGATING)
        public DeliveryData(String status, String estimatedArrival) {
            this.status = status;
            System.out.println(status);
            this.estimatedArrival = estimatedArrival;
            isDelivered = status.equals("Lieferung erfolgreich");
        }

        public boolean isDelivered() {
            return isDelivered;
        }

        public String getStatus() {
            return status;
        }

        public String getEstimatedArrival() {
            return estimatedArrival;
        }
    }

}
